"use strict";
Object.defineProperty(exports, "__esModule", {
    value: true
});
Object.defineProperty(exports, "SchedulerProcessor", {
    enumerable: true,
    get: ()=>SchedulerProcessor
});
const _aliceBob = require("alice-bob");
const _syncedSet = require("synced-set");
const _schedulerCore = require("./scheduler-core");
const _util = require("./util");
const _clock = require("./clock");
let lastReceivedTime = currentTime;
let SchedulerProcessor = class SchedulerProcessor extends AudioWorkletProcessor {
    running = false;
    clock = new _clock.Clock();
    blockSize = 128;
    blockTime = this.blockSize / sampleRate;
    diffTime = 0;
    eventGroups = new _syncedSet.SyncedSet({
        send: (_, cb)=>cb(),
        pick: _schedulerCore.core.pickFromLocal,
        reducer: (eventGroup)=>({
                targets: eventGroup.targets
            }),
        equal: (prev, next)=>prev.targets === next.targets
    });
    suspended = new Set();
    turns = new Map();
    node;
    constructor(){
        super();
        const [worklet, node] = new _aliceBob.Bob((data)=>this.port.postMessage({
                rpc: data
            }), this).agents({
            debug: false
        });
        this.node = node;
        this.port.onmessage = ({ data  })=>{
            if (data.rpc) worklet.receive(data.rpc);
            if (data.eventGroups) this.eventGroups.receive(_schedulerCore.core.deserialize(data.eventGroups));
        };
        this.eventGroups.on('add', (eventGroup)=>{
            this.turns.set(eventGroup, new Map());
            this.requestNextEvents(eventGroup.id, 0, 2);
        });
        this.eventGroups.on('delete', (eventGroup)=>{
            this.turns.delete(eventGroup);
        });
    }
    setBpm(bpm) {
        this.clock.coeff = bpm / (60 * 4);
        this.blockTime = this.blockSize / sampleRate * this.clock.coeff;
    }
    setClockBuffer(clockBuffer) {
        this.clock.buffer = clockBuffer;
    }
    start(playbackStartTime = currentTime, offsetStartTime = 0) {
        this.running = true;
        this.diffTime = playbackStartTime - currentTime;
        this.clock.internalTime = -this.diffTime * this.clock.coeff + offsetStartTime;
        lastReceivedTime = currentTime;
    }
    stop() {
        this.running = false;
    }
    suspendTarget(targetId) {
        out: for (const eventGroup of this.eventGroups){
            for (const target of eventGroup.targets){
                if (target.id === targetId) {
                    target.midiQueue.clear();
                    break out;
                }
            }
        }
        this.suspended.add(targetId);
    }
    resumeTarget(targetId) {
        this.suspended.delete(targetId);
    }
    clearEventGroup(eventGroupId) {
        const eventGroup = [
            ...this.eventGroups
        ].find((eventGroup)=>eventGroup.id === eventGroupId);
        this.waitingEvents.clear();
        this.turns.get(eventGroup)?.clear();
    }
    waitingEvents = new Set();
    receiveEvents(eventGroupId, turn, turns, clear) {
        for(let i = turn; i < turn + turns.length; i++){
            this.waitingEvents.delete(`${[
                eventGroupId,
                i
            ]}`);
        }
        const eventGroup = [
            ...this.eventGroups
        ].find((eventGroup)=>eventGroup.id === eventGroupId);
        if (!eventGroup) {
            console.warn(`Event group with id "${eventGroupId}" not found`);
            return;
        }
        const map = this.turns.get(eventGroup);
        if (clear) map.clear();
        for(let i1 = turn, x = 0; i1 < turn + turns.length; i1++){
            map.set(i1, turns[x++]);
        }
    }
    process() {
        if (!this.running) return true;
        const now = currentTime;
        const elapsedTime = now - lastReceivedTime;
        lastReceivedTime = now;
        const prevInternalTime = this.clock.internalTime;
        const addTime = elapsedTime * this.clock.coeff;
        let internalTime = prevInternalTime + addTime;
        this.clock.internalTime = internalTime;
        if (prevInternalTime < 0 && this.clock.internalTime > 0) {
            internalTime = this.clock.internalTime = 0;
            this.clock.offsetFrame = currentFrame;
        }
        for (const eventGroup of this.eventGroups){
            const { needTurn , results: events  } = (0, _util.getEventsInRange)(internalTime, this.blockTime, eventGroup, this.turns.get(eventGroup));
            for (const target of eventGroup.targets){
                if (this.suspended.has(target.id)) continue;
                for (const [receivedTime, event] of events){
                    target.midiQueue.push(receivedTime, ...event.midiEvent.data);
                }
            }
            if (needTurn >= 0 && (!this.waitingEvents.has(`${[
                eventGroup.id,
                needTurn
            ]}`) || !this.waitingEvents.has(`${[
                eventGroup.id,
                needTurn + 1
            ]}`)) && (!this.turns.get(eventGroup).has(needTurn) || !this.turns.get(eventGroup).has(needTurn + 1))) {
                this.requestNextEvents(eventGroup.id, needTurn, 2);
            }
        }
        return true;
    }
    requestNextEvents(eventGroupId, turn, total) {
        try {
            this.node.requestNextEvents(eventGroupId, turn, total);
            for(let i = 0; i < total; i++){
                this.waitingEvents.add(`${[
                    eventGroupId,
                    turn + i
                ]}`);
            }
        } catch (error) {
            console.warn(error);
        }
    }
};
registerProcessor('scheduler', SchedulerProcessor);

//# sourceMappingURL=scheduler-processor.js.map